@@ -1,7 +1,10 @@
-`FitARp` <-
+FitARp <-
 function(z,p,lag.max="default",MLEQ=FALSE)
 {
 stopifnot(length(z)>0, length(z)>length(p), length(p)>0)
+is.wholenumber <-
+    function(x, tol = .Machine$double.eps^0.5)  abs(x - round(x)) < tol
+stopifnot(is.wholenumber(p), p>0)
 n<-length(z)
 if (lag.max=="default")
     MaxLag <- min(300, ceiling(length(z)/5))
@@ -80,3 +83,4 @@ ans<-list(loglikelihood=ans$loglikelihood,phiHat=phiHat,sigsqHat=sigsq,muHat=mz,
 class(ans)<-"FitAR"
 ans
 }
+
