@@ -1,12 +1,15 @@
-`FitARz` <-
+FitARz <-
 function (z, p, demean = TRUE, MeanMLEQ = FALSE, lag.max = "default") 
 {
     stopifnot(length(z) > 0, length(z) > max(p), length(p) > 
         0)
+    is.wholenumber <-
+    function(x, tol = .Machine$double.eps^0.5)  abs(x - round(x)) < tol
+    stopifnot(is.wholenumber(p), p>0)
     ztsp <- tsp(z)
     if (lag.max == "default") 
-    	MaxLag <- min(300, ceiling(length(z)/5))
-    else 	MaxLag = lag.max
+        MaxLag <- min(300, ceiling(length(z)/5))
+    else    MaxLag = lag.max
     MaxIter <- 10
     n <- length(z)
     pvec <- sort(p)
